/* 
Programm fr LED-THERMOMETER
--------------------------
Beschreibung: Programm fr Thermometer
Erstellt von: Thomas Moosmann
Erstellt am:  12.08.2006 (Version 1.5)
System(e): Tiny26 (interner RC-Oszillator mit 1MHz)

BEMERKUNG: KEIN Testmodus vorhanden!
*/


// Konfigurationsvariablen
/* 
   Korrekturwert fr Temperaturmessung zum Abgleichen 
   der Temperatur mit externem Thermometer
*/
char korrektur_temperaturdifferenz=10; //Beispiel 10= 1,0C!)
/*
Der Wert gibt an, nach welcher Zeit der die aktuell gemessene Temperatur 
am Display ausgegeben wird.
*/
char anzeige_aktualisieren_zeit=60; //Beispiel 60 = 60 Sekunden


// Globale Variablen sowie wichtige Include-Dateien fr das Program
#include <iot26v.h>
#include <macros.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

char anzeigetext[5]; // Variable fr Ausgabe der aktuellen Zeit ber LED-Display
char timer0_sekunden_takt;  // Variable fr den Sekundentakt des Thermometers
char minuten_takt=0; // Variable fr den Minutentakt der Stoppuhr
char timer0_takt=0;  // Variable fr den TIMER0 des Mikrocontrollers
char adc_wert=0; // Variable mit dem aktuellen Wert des Analog-Digital-Wandlers


// Einstellungen fr Display
// PORTB-WERTE fr ZAHLEN-DARSTELLUNG
char null = 0x40;
char eins = 0x79;
char zwei = 0x24;
char drei = 0x30;
char vier = 0x19;
char fuenf =0x12;
char sechs =0x02; 
char sieben=0x78;
char acht = 0x00;
char neun = 0x10;
char c_celcius = 0x46; // Hex-Wert um "C" am Display auszugeben!
char c_grad= 0x1C;// Hex-Wert um "" am Display auszugeben!

// PORTA-WERTE fr 7-SEGMENT-ANZEIGE
char stelle4=0xE0;
char stelle3=0xD0;
char stelle2=0xB0;
char stelle1=0x70;


//ANALOG-DIGITAL-WANDLER wertet die Spannung des LM35 aus
/*
 ADC initialize
 Conversion time: 1664uS; 
 Umwandlung: Freeruning (dauernd)
 Uref = Interne Referenzspannung ?
 ADC0 = Ausgangsspannung des LM35 entspricht Temperatur
*/
void adc_init(void)
{
 ADCSR = 0x00;
 ADCMUX = 0x00;
 ACSR  = 0x80;
 ADCSR = 0xEF;
}

#pragma interrupt_handler adc_isr:12
void adc_isr(void)
{
 /* Liest das Register des AD-Wandlers aus. */
 adc_wert=ADCL; 
 adc_wert|=(int)ADCH << 8;
 /* Thermometer soll nur GANZE Gradwerte anzeigen
    BIT0 (= 0,5C)der Variablen 'adc_wert' wird immer auf Null gesetzt*/
 adc_wert &= ~(1 << 0);  
}


//SOFTWARE-TIMER TIMER0 erzeugt Sekundentakt
//TIMER0 initialize - prescale:1024 // actual value: 19,465mSec (0,3%)
void timer0_init(void)
{
 TCCR0 = 0x00; //stop timer
 TCNT0 = 0xD9; //set count
 TCCR0 = 0x05; //start timer
}

#pragma interrupt_handler timer0_ovf_isr:7
void timer0_ovf_isr(void)
{
 TCNT0 = 0xED;
 timer0_takt++;
 
 if (timer0_takt==52)
    {
	 timer0_sekunden_takt++;
	 timer0_takt=0;
    }
}


//ZEITVERZGERUNG FR ANSTEUERUNG (MULTIPLEXEN) DER 4 STELLEN
// 8ms! @ Tiny26 mit 1MHz //
void delay() 
{
	char delay_schleife=1;
	short delay_wert=0;
	
    while(delay_schleife)
	{
	if (delay_wert==432){delay_schleife=0;} // Standardwert = 432
	delay_wert++;
	}
}


// INITIALISIERUNG DER PORTS
// EINSTELLUNG DER PORTS ALS I/Os
void port_init(void)
{
 PORTB = 0x00;
 DDRB  = 0x7F;
 PORTA = 0x00;
 DDRA  = 0xF2;
}


// STARTROUTINE DES MIKROCONTROLLERS
void init_devices(void)
{
 
 CLI();
 port_init();
 timer0_init();
 adc_init();
 
 MCUCR = 0x00;
 TIMSK = 0x02; 
 GIMSK = 0x00;
 PLLCSR = 0x00;
 SEI();
 
}


/* ANZEIGE-ROUTINE FR MULTIPLEXTE 4x7-SEGMENTANZEIGE
   a.) wandelt den ASCII-Wert in Siebensegment-Code um
   b.) fgt kleineren Temperaturen fhrende Nullen hinzu
       so ergibt sich statt: 9C --> 09C
   c.) Schaltet so schnell zwischen den Ziffern um, das 
       das Auge ein stehendes Bild wahrnimmt. Frequenz ca. 31 Hz
*/
void siebensegmentanzeige(void)
{
char einenull[5]="0";
char gradcelcius[5]="C";
char multiplexen=1;
char laenge=0;
char ziffer=0;
char asciiwert;

laenge = strlen(anzeigetext);
laenge--;

//Bearbeitung fr Temperaturen <=9C ==> 09C!
if (laenge==0)
    {
	strcat(einenull,anzeigetext);   // fgt Temperatur eine fhrende Null hinzu
	strcpy(anzeigetext,einenull);   // kopiert Variablen um.
	strcat(anzeigetext,gradcelcius);// fgt der Temperatur "C" hinzu.
	}
	
//Bearbeitung fr Temperaturen >10C ==> 10C!
if (laenge==1)
    {
	 strcat(anzeigetext,gradcelcius); // fgt der Temperatur "C" hinzu.
	}

laenge = strlen(anzeigetext);
laenge--;

while(multiplexen)
{
asciiwert=anzeigetext[laenge];
if (asciiwert==48){PORTB=null;}
if (asciiwert==49){PORTB=eins;}		
if (asciiwert==50){PORTB=zwei;}		
if (asciiwert==51){PORTB=drei;}		
if (asciiwert==52){PORTB=vier;}		
if (asciiwert==53){PORTB=fuenf;}		
if (asciiwert==54){PORTB=sechs;}		
if (asciiwert==55){PORTB=sieben;}		
if (asciiwert==56){PORTB=acht;}		
if (asciiwert==57){PORTB=neun;}
if (asciiwert==67){PORTB=c_celcius;}
if (asciiwert==176){PORTB=c_grad;}
if (ziffer == 3){PORTA=stelle4; delay();}
if (ziffer == 2){PORTA=stelle3; delay();}
if (ziffer == 1){PORTA=stelle2; delay();}
if (ziffer == 0){PORTA=stelle1; delay();}
if (laenge==0){multiplexen=0;}
laenge--;
ziffer++;
}
}


/* HAUPT-ROUTINE DES THERMOMETERS
a.) Gibt beim Start den 00C aus = OK!
b.) Die Hauptroutine: 
    1.) Rechnet die ADC-Werte in Dezimalen um und gibt sie mit 
        dem Format: xxC auf der Anzeige aus (26C = 26,0C).
    2.) Die Anzeige wird kontinuierlich nach einer bestimmten Zeit aktualisiert.
	3.) Die rote LED leuchtet immer kurzzeitg auf, wenn die Temperatur gemessen
	    und damit auch die angezeigte Temperatur gemessen wird.
*/

void main()
{
 int anzeigeasciiwert=0;
 init_devices();
 itoa(anzeigetext,0,10);
 while(1)
  {
   siebensegmentanzeige();
   if (timer0_sekunden_takt==anzeige_aktualisieren_zeit)
       {
	    PORTA |= (1 << 1);    // rote LED (PA1) an ==> Anzeige wird aktualisiert
	   }
   if (timer0_sekunden_takt==(anzeige_aktualisieren_zeit+1))
       {
	    timer0_sekunden_takt=0; 
		anzeigeasciiwert=(((adc_wert*5)-korrektur_temperaturdifferenz)/10);
		PORTA &= ~(1 << 1);  // rote LED (PA1) aus => Anzeige wurde aktualisiert
	   }
   itoa(anzeigetext,anzeigeasciiwert,10); //itoa wandelt Zahl in ASCII um
  }
}
